package com.atlassian.confluence.test.rpc;

import bucket.user.BucketHibernateConfigProvider;
import bucket.user.UserAccessor;
import bucket.cache.CacheManager;
import com.atlassian.bandana.BandanaManager;
import com.atlassian.confluence.core.ContentPropertyManager;
import com.atlassian.confluence.core.ContentEntityManager;
import com.atlassian.confluence.core.ContentEntityObject;
import com.atlassian.confluence.pages.Attachment;
import com.atlassian.confluence.pages.AttachmentManager;
import com.atlassian.confluence.pages.Page;
import com.atlassian.confluence.pages.PageManager;
import com.atlassian.confluence.rpc.NotPermittedException;
import com.atlassian.confluence.rpc.RemoteException;
import com.atlassian.confluence.search.lucene.ConfluenceIndexManager;
import com.atlassian.confluence.security.CaptchaManager;
import com.atlassian.confluence.security.Permission;
import com.atlassian.confluence.security.PermissionManager;
import com.atlassian.confluence.setup.bandana.ConfluenceBandanaContext;
import com.atlassian.confluence.spaces.Space;
import com.atlassian.confluence.spaces.SpaceManager;
import com.atlassian.confluence.spaces.SpaceType;
import com.atlassian.confluence.themes.ThemeManager;
import com.atlassian.confluence.user.AuthenticatedUserThreadLocal;
import com.atlassian.confluence.user.PersonalInformationManager;
import com.atlassian.confluence.util.GeneralUtil;
import com.atlassian.event.Event;
import com.atlassian.spring.container.ComponentNotFoundException;
import com.atlassian.spring.container.ContainerManager;
import com.atlassian.user.Group;
import com.atlassian.user.User;
import com.atlassian.user.impl.cache.Cache;
import com.atlassian.user.search.page.Pager;
import com.opensymphony.module.propertyset.PropertySet;
import org.apache.log4j.Category;
import org.hsqldb.lib.StringInputStream;
import org.quartz.Scheduler;

import java.util.*;

public class FuncTestRpcHandlerImpl implements FuncTestRpcHandler
{
    public static final Category log = Category.getInstance(FuncTestRpcHandlerImpl.class);

    private ConfluenceIndexManager indexManager;
    private SpaceManager spaceManager;
    private PermissionManager permissionManager;
    private UserAccessor userAccessor;
    private AttachmentManager attachmentManager;
    private PageManager pageManager;
    private ThemeManager themeManager;
    private CaptchaManager captchaManager;
    private FuncTestEventQueue funcTestEventQueue;
    private Scheduler scheduler;
    private BandanaManager bandanaManager;
    private BucketHibernateConfigProvider configProvider;
    private ContentPropertyManager contentPropertyManager;
    private ContentEntityManager contentEntityManager;
    private PersonalInformationManager personalInformationManager;
    private CacheManager cacheManager;

    public boolean logMessage(String token, String destination, String message) throws NotPermittedException, RemoteException
    {
        assertCanAdminister();

        if ("sout".equalsIgnoreCase(destination))
            System.out.println(message);
        else if ("serr".equalsIgnoreCase(destination))
            System.err.println(message);
        else if ("info".equalsIgnoreCase(destination))
            log.info(message);
        else if ("debug".equalsIgnoreCase(destination))
            log.debug(message);
        else if ("warn".equalsIgnoreCase(destination))
            log.warn(message);
        else if ("error".equalsIgnoreCase(destination))
            log.error(message);

        else
            return false;

        return true;
    }

    private void assertCanAdminister() throws NotPermittedException
    {
        User user = AuthenticatedUserThreadLocal.getUser();

        if (!permissionManager.hasPermission(user, Permission.ADMINISTER, PermissionManager.TARGET_APPLICATION))
        {
            throw new NotPermittedException("You don't have administration permission.");
        }
    }

    public boolean nukeForTests(String token, Vector usersToKeep, Vector groupsToKeep, Vector spaceKeysToKeep) throws NotPermittedException, RemoteException
    {
        assertCanAdminister();

        // remove all spaces
        List spaces = spaceManager.getAllSpaces();
        for (Iterator iterator = spaces.iterator(); iterator.hasNext();)
        {
            Space space = (Space) iterator.next();
            if (spaceKeysToKeep.contains(space.getKey()))
                continue;

            spaceManager.removeSpace(space);
        }

        // remove all groups
        Pager groups = userAccessor.getGroups();
        for (Iterator iterator = groups.iterator(); iterator.hasNext();)
        {
            Group group = (Group) iterator.next();
            if (groupsToKeep.contains(group.getName()))
                continue;

            userAccessor.removeGroup(group);
        }

        // remove any personal spaces (until removeUser does it for us)
        List pspaces = spaceManager.getSpacesByType(SpaceType.PERSONAL);
        for (Iterator iterator = pspaces.iterator(); iterator.hasNext();)
        {
            Space pSpace = (Space) iterator.next();
            spaceManager.removeSpace(pSpace);
        }

        // remove all users
        Pager users = userAccessor.getUsers();
        for (Iterator iterator = users.iterator(); iterator.hasNext();)
        {
            User user = (User) iterator.next();
            if (usersToKeep.contains(user.getName()))
                continue;

            userAccessor.removeUser(user);
            logMessage(token, "sout", "removed user " + user.getName());
        }

        // flush the index queue
        indexManager.flushQueue();

        captchaManager.setCaptchaEnabled(false);

        return true;
    }

    public boolean massCreateAttachments(String token, String pageId, String prefix, String numAttachments) throws Exception
    {
        assertCanAdminister();

        Page p = pageManager.getPage(Long.parseLong(pageId));

        for (int i = 0; i < Integer.parseInt(numAttachments); i++)
        {
            Attachment a = new Attachment();
            a.setFileName(prefix + i + ".txt");
            a.setContentType("text/plain");
            a.setContent(p);
            p.addAttachment(a);

            String attachmentContent = "Medium size attachment content";

            if (i % 10 == 0)
                attachmentContent = "This attachment is slightly larger than any other attachment";
            else if (i % 5 == 0)
                attachmentContent = "Small content";

            a.setFileSize(attachmentContent.length());
            attachmentManager.saveAttachment(a, null, new StringInputStream(attachmentContent));
        }

        return true;
    }

    public boolean resetTheme(String token) throws Exception
    {
        assertCanAdminister();
        themeManager.setGlobalTheme(null);
        return true;
    }

    public Vector getRecentEvents(String token) throws Exception
    {
        List events = getFuncTestEventQueue().getEvents();
        Vector eventNames = new Vector();

        for (int i = 0; i < events.size(); i++)
        {
            Event event = (Event) events.get(i);
            eventNames.add(event.getClass().toString());
        }
        return eventNames;
    }

    public boolean checkSpringBean(String token, String name)
    {
        return ContainerManager.getComponent(name) != null;
    }

    public boolean checkNoSpringBean(String token, String name)
    {
        try
        {
            return ContainerManager.getComponent(name) == null;
        }
        catch (ComponentNotFoundException e)
        {
            // We're actually expecting a CNFE.
            if (e.getMessage().indexOf("No bean named '" + name + "' is defined") > -1)
                return true;
            // I seriously doubt this will ever happen.
            throw e;
        }
    }


    public boolean shutdownQuartz(String token) throws Exception
    {
        scheduler.shutdown();
        return true;
    }

    public boolean removeBandanaKey(String token, String space, String key)
    {
        bandanaManager.setValue(GeneralUtil.stringSet(space) ? new ConfluenceBandanaContext(space) : new ConfluenceBandanaContext(), key, "");
        return true;
    }

    public Vector getContentPropertyKeys(String token, String entityId)
    {
        Map args = new HashMap();
        args.put("entityId", new Long(entityId));
        args.put("entityName", "confluence_ContentEntityObject");
        args.put("configurationProvider", getConfigProvider());
        PropertySet propertySet = com.opensymphony.module.propertyset.PropertySetManager.getInstance("bucket", args);

        return new Vector(propertySet.getKeys());
    }

    public boolean setContentTextProperty(String token, String entityId, String key, String value)
    {
        ContentEntityObject entity = contentEntityManager.getById(Long.parseLong(entityId));
        contentPropertyManager.setTextProperty(entity, key, value);
        return true;
    }

    public String getPersonalInformationId(String token, String username) {
        return personalInformationManager.getPersonalInformation(username).getIdAsString();
    }

    public boolean putStringMappingInCache(String token, String cacheName, String key, String value)
    {
        Cache cache = cacheManager.getCache(cacheName);
        cache.put(key, value);
        return true;
    }

    public boolean flushAllCaches(String token)
    {
        cacheManager.flushCaches();
        return true;
    }


    private BucketHibernateConfigProvider getConfigProvider()
    {
        if (configProvider == null)
            configProvider = new BucketHibernateConfigProvider();

        return configProvider;
    }

    public void setIndexManager(ConfluenceIndexManager indexManager)
    {
        this.indexManager = indexManager;
    }

    public void setSpaceManager(SpaceManager spaceManager)
    {
        this.spaceManager = spaceManager;
    }

    public void setPermissionManager(PermissionManager permissionManager)
    {
        this.permissionManager = permissionManager;
    }

    public void setUserAccessor(UserAccessor userAccessor)
    {
        this.userAccessor = userAccessor;
    }

    public void setAttachmentManager(AttachmentManager attachmentManager)
    {
        this.attachmentManager = attachmentManager;
    }

    public void setPageManager(PageManager pageManager)
    {
        this.pageManager = pageManager;
    }

    public void setThemeManager(ThemeManager themeManager)
    {
        this.themeManager = themeManager;
    }

    public void setCaptchaManager(CaptchaManager captchaManager)
    {
        this.captchaManager = captchaManager;
    }

    public void setFuncTestEventQueue(FuncTestEventQueue funcTestEventQueue)
    {
        this.funcTestEventQueue = funcTestEventQueue;
    }

    public void setScheduler(Scheduler scheduler)
    {
        this.scheduler = scheduler;
    }

    public FuncTestEventQueue getFuncTestEventQueue()
    {
        return funcTestEventQueue;
    }

    public void setBandanaManager(BandanaManager bandanaManager)
    {
        this.bandanaManager = bandanaManager;
    }

    public void setContentPropertyManager(ContentPropertyManager contentPropertyManager)
    {
        this.contentPropertyManager = contentPropertyManager;
    }

    public void setContentEntityManager(ContentEntityManager contentEntityManager)
    {
        this.contentEntityManager = contentEntityManager;
    }

    public void setPersonalInformationManager(PersonalInformationManager personalInformationManager)
    {
        this.personalInformationManager = personalInformationManager;   
    }

    public void setCacheManager(CacheManager cacheManager)
    {
        this.cacheManager = cacheManager;
    }
}
